#!/usr/bin/env python

# File history:
#
# 14.01.2010:
# Bug fix for the treatment of the query boxsize. This parameter
# was not taken into account properly before.
#
# 25.08.2010:
# The observation date can be obtained with the key 'creation_date',
# not with obsdate.

"""
SCRIPT NAME:
    CFHT_dataquery.py - query the CFHT data archive at CADC

SYNOPSIS:
    CFHT_dataquery.py key value .. key value prefix_name

DESCRIPTION:
    The script is an interface for queries of CFHT observations
    at CADC. It replaces WWW-Browser queries to the CADC service
    at http://www3.cadc-ccda.hia-iha.nrc-cnrc.gc.ca/cfht/ and returns
    HTML pages as result.
    The script call consists of CADC archive key/value query pairs. The
    last argument gives the prefix for the output file name; the final
    file name is 'prefix_cfht.html'. All keys are predefined with
    'reasonable' default values and hence not all keys need to be
    apecified.
    The possible keys are listed in the 'CADC QUERY KEYS' section below.
    The resulting HTML file can be postprocessed with other utilities, e.g.
    the 'extract_CFHT_info.sh' script. For the syntax of the different keys
    see the CADC WWW pages and also the EXAMPLES section below.
         
CADC QUERY KEYS:
                 box: 00 30 00           (search box around ra, dec position)
            category: CAL                (RAW: raw data; CAL: Elixir data) 
       creation_date: not constrained    (data creation/observation date) 
                 dec: +52:00:00          (object declination)  
              expnum: not constrained    (CFHT exposure number, e.g. 780943) 
            exposure: not constrained    (???) 
              filter: r                  (optical filter) 
          instrument: MEGAPRIME          (MEGAPRIME, 'CFH12K MOSAIC')  
   max_rows_returned: 5000               (maximum of returned entries) 
             obstype: OBJECT             (OBJECT, FLAT, BIAS, DARK) 
         photometric: %                  (yes or no; photometric data?) 
             pi_name: not constrained    (name of Principal Investigator) 
                  ra: 14:08:00           (object right ascension) 
        release_date: not constrained    (release date for data) 
               runid: not constrained    (CFHT RUNID) 
              simbad: simbad             (query data base for 'tobject' entry) 
      tab_instrument: on                 (???) 
             tobject: not constrained    (Simbad Target Name) 
          wdbi_order: not constrained    (???) 

EXAMPLES:
   - Query data in a 1 sq. degree box around the position
     ra=14:08:00, dec=+52:00:00

     CFHT_dataquery.py ra 14:08:00 dec +52:00:00 box "01 00 00" test

     The result will go to test_cfht.html

   - Query all CFH12K data around the cluster A1689:

     CFHT_dataquery.py tobject A1689 instrument "CFH12K MOSAIC" A1689

     The result will go to A1689_cfht.html
     
   - Query all Elixir processed images from the P.I. Hudson

     CFHT_dataquery.py  pi_name Hudson category CAL HUDSON

   - Query all R-Band CFH12K Flat fields from first of February 2000 to
     first of April 2000

     CFHT_dataquery.py filter R creation_date "02/01/2000 .. 04/01/2000"
                       obstype FLAT instrument "CFH12K MOSAIC" FLAT 

AUTHOR:
    Thomas Erben         (terben@astro.uni-bonn.de)
    
    The script is heavily based on Peter Erwin's 'telarchive' tools
    (see http://www.mpe.mpg.de/~erwin/code/).
"""

import sys
import te_archive_search
import te_archive_list
import te_cfht_archive

if (len(sys.argv) - 2) % 2 == 1 or len(sys.argv) < 2:
    print "wrong command line syntax:"
    print __doc__
    sys.exit(1)

# The number of key/value pairs in our argument list:    
nkeys = (len(sys.argv) - 2) / 2

# dummy initialisations to satisfy 'telarchive'
# initialisation requirements:
whichHSTarchive = 0
doAAT = 0
doSDSS = False
archive_module_list = [ "te_cfht_archive" ]
coordsList = []
# The target name here is only used for the name of the output
# file:
targetName = sys.argv[len(sys.argv) - 1]

# we need to modify the CFHT archive dictionary
# BEFORE an instance of ArchiveList is created 
# below:
for i in range(1, nkeys + 1):
    if sys.argv[2 * i - 1] in te_cfht_archive.DICT.keys():
        te_cfht_archive.DICT[sys.argv[2 * i - 1]] = sys.argv[2 * i]
    else:
        print "%s is not a query possibility!" % (sys.argv[2 * i - 1])
        print __doc__
        sys.exit(1)

# create an instance of ArchiveList
#
# NOTE that the third argument is here a string with the
# "boxstring" as CFHT needs it. In Peters original code, this
# is a float which represents the search radius in arcmins.
# For this special treatment here, we also overrode the
# InsertBoxSize method in the CFHT class!
theArchiveList = te_archive_list.ArchiveList( targetName, coordsList,
                                              te_cfht_archive.DICT['box'],
			   		      whichHSTarchive, doAAT, doSDSS,
					      archive_module_list )

print "Your CADC query parameters:"
print "==========================="
print ""
for key in sorted(te_cfht_archive.DICT.iterkeys()):
    if te_cfht_archive.DICT[key] == "":
        print "%20s: %-20s" % (key, "not constrained")
    else:
        print "%20s: %-20s" % (key, te_cfht_archive.DICT[key])
print ""
print "-------------------------------------------------"
print "Query Results:"

doDebug = 1     # output an HTML file with all the found data!
                # the name of that file is 'targetName'_cfht.html
doThreading = 0 # treating with one archive we do not need threading

# finally do the archive search:
te_archive_search.DoSearch( targetName, theArchiveList, doDebug, doThreading )

# bye
sys.exit(0)
