#!/bin/bash -u

# ----------------------------------------------------------------
# File Name:           extract_CFHT_info.sh           
# Author:              Thomas Erben (terben@astro.uni-bonn.de)
# Last modified on:    23.03.2010
# Description:         postprocess HTML output from CFHT_dataquery.py
# ----------------------------------------------------------------

# SCRIPT HISTORY
# ==============
#
# 22.01.2010:
# I included the exposure date in the output.
#
# 26.01.2010:
# I included Ra and Dec in the output
#
# 23.03.2010:
# The HTML input file from CFHT queries changed and we adapted this
# script accordingly!

function printUsage 
{
  echo "SCRIPT NAME:"
  echo "    extract_CFHT_info.sh - postprocess HTML output from CFHT_dataquery.py"
  echo ""
  echo "ARGUMENTS:"
  echo "    extract_CFHT_info.sh input_html output_ascii"
  echo ""
  echo "DESCRIPTION:"
  echo "    The script can be run on HTML output from the CFHT_dataquery.py"
  echo "    script. It extracts image_name, object, Ra, Dec, filter,"
  echo "    exposure, instrument, the name of the P.I. and the exposure date."
  echo "    It formats the output to columns separated by '|'. "
  echo "    The script does NOT check whether the input really originates"
  echo "    from CFHT_dataquery.py."
  echo ""
  echo "EXAMPLES:"
  echo "    Query data in a 1 sq. degree box around the position"
  echo "    ra=14:08:00, dec=+52:00:00 and save the formatted output"
  echo "    in 'output.txt':"
  echo ""
  echo "    - CFHT_dataquery.py ra 14:08:00 dec +52:00:00 box "01 00 00" out"
  echo "    - extract_CFHT_info.sh out_cfht.html output.txt"
  echo ""
  echo "AUTHOR:"
  echo "    Thomas Erben         (terben@astro.uni-bonn.de)"
}

# sanity checks:
if [ $# -ne 2 ]; then
  printUsage
  exit 1;
fi

if [ ! -s $1 ]; then
  echo "File $1 does not exist or is empty!"
  exit 1;
fi

# here the script work starts:
grep "<TD NOWRAP ALIGN=CENTER>" $1 |\
tr -s '<>' '\n' |\
 awk 'BEGIN {startblock = 1} { 
      if (NR == (startblock + 2)) {
        object = $0;
      } 
      if (NR == (startblock + 7)) {
        split($0, a)
        ra = a[1] ":" a[2] ":" a[3]
      }
      if (NR == (startblock + 12)) {
        split($0, a)
        dec = a[1] ":" a[2] ":" a[3]
      }
      if (NR == (startblock + 22)) {
        image = $0;
      }
      if (NR == (startblock + 27)) {
        instrument = $0;
      } 
      if (NR == (startblock + 31)) {
        exptime = $0;
      } 
      if (NR == (startblock + 35)) {
        filter = $0;
      } 
      if (NR == (startblock + 53)) {
        pi = $0;
      } 
      if (NR == (startblock + 48)) {
        expdate = $0;
      } 
      if (NR == (startblock + 61)) {
        imtype = $0;
        printf("%10s | %20s | %10s | %10s | %10s | %10s | %15s | %20s | %20s |\n", 
               image, object, ra, dec, filter, exptime, 
               instrument, pi, expdate);
        startblock += 67; 
      }}' > $2

